/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/



#ifndef _ULDB_H_    // make sure we only include this code once
#define _ULDB_H_

#include "ulglobal.h"

/** The maximum length of a customer's first name.
 */
#define FIRST_NAME_LEN   (20)
/** The maximum length of a customer's last name.
 */
#define LAST_NAME_LEN    (30)

/** The maximum length of an error message.
 */
#define ERROR_MSG_LEN        (120)

/** The maximum length of a username which is used for synchronization.
 */
#define MAX_SYNCH_USER_NAME_LEN    (10)

/** The maximum length of the version string which is used for synchronization.
 */
#define MAX_SYNCH_VERSION_LEN      (20)

/** The maximum length of the stream parameters which are used synchronization.
 */
#define MAX_SYNCH_STREAM_PARMS_LEN (1024)

/** A structure which contains information about a given customer.
 */
struct customer_struct {

	/** Represents the customer's ID number.
	 */
	long  customer_id;

	/** Represents the customer's first name.
	 */
	TCHAR first_name[ FIRST_NAME_LEN + 1 ];

	/** Represents the customer's last name.
	 */
	TCHAR last_name[ LAST_NAME_LEN + 1 ];
};
// TODO: this should be a C++ class instead of a struct...

/** An enumeration of possible startup values for the database.
 */
enum DB_STARTUP_STATUS {

	/** db_init() was called on startup, and all is OK.
	 */
	DB_STARTUP_SUCCESS_FIRST,

	/** db_init() was not called on startup, and all is OK.
	 */
	DB_STARTUP_SUCCESS,

	/** ULPalmLauch() failed. This only occurs on Palm.
	 */
	DB_STARTUP_FAIL_PALM_LAUNCH, 

	/** db_init() was called, however it encountered na error.
	 */
	DB_STARTUP_FAIL_DB_INIT,

	/** Failed to connect to the UltraLite database.
	 */
	DB_STARTUP_FAIL_CONNECT

};

/** The class which performs the access to the UltraLite Database.
	
    This class will contain all of the embedded SQL statements needed
    for ULFoundation, and can be used on multiple platforms (Palm,
    VxWorks, Windows CE, etc.)  without any modifications. By
    constructing the program in this manner, the data access classes
    are isolated from the GUI and therefore the time required to port
    the UltraLite program between platforms is minimized.

  */
class ULDB {
 public:

	/** The constructor for the UltraLite Database connection.
		
	    This initializes the information required for connecting
	    to the database, such as the method of connection used and
	    the Mobilink server to synchronize with.

	    @param sync_user	Username for synchronization.
	    @param sync_version	Version of the synchronization scripts to use.
	    @param stream	Connection stream to synchronize through.
	    @param stream_parms	Parameters for the connection stream.

	 */
	ULDB( TCHAR *sync_user, TCHAR *sync_version, ul_stream_defn *stream,
		  TCHAR *stream_parms );

	/** Cleans up the database connection.
	 */
	~ULDB( void );

	// UltraLite administration functions
	//

	/** Sets the username with which to synchronize to Mobilink.

		@param synch_user The username used to synchronize with Mobilink.
	 */
	void SetSynchUserName( TCHAR *synch_user );

	/** Sets the version of the synchronization scripts for Mobilink to use.
		
		@param synch_version The version of the synchronization scripts to use.
	 */
	void SetSynchVersion( TCHAR *synch_version );

	/** Sets any connection stream parameters necessary to synchronize.

		@param stream_parms	Any necessary connection stream parameters.
	 */
	void SetSynchStreamParms( TCHAR *stream_parms );

	/** Initializes and connects to the Ultralite Database.

		@return A <code>DB_STARTUP_STATUS</code> which represents the result of
		the startup attempt.
	 */
	DB_STARTUP_STATUS  StartupDB( void );

	/** Performs synchronization between the Ultralite database and the Mobilink server.

		@return The long int representation of the SQLCODE returned by Mobilink.
	 */
	long Synchronize( void );

#ifdef UNDER_PALM_OS

	/** Shut down the database engine.
		
		If this is running on Palm, we want to return a ul_bool which represents the
		result of the shutdown attempt.

		@return A ul_bool which returns false (0) if an error occurs, true otherwise.
	 */
	ul_bool ShutdownDB( void );

#else

	/** Shut down the database engine.
		
		If this is running on some tplatform other than Palm, we want to return
		a ul_bool which represents the result of the shutdown attempt.

		@return An unsigned short int which returns 0 if an error occurs, true otherwise.
	 */
	unsigned short ShutdownDB( void );

#endif

	// Customer table functions

	/** Open a cursor which scans through the Customer table.

		This method must be called before GetNextCustomerRecord is called for
		the first time, or an error will likely result. It will initialize the
		cursor to point at hte first row in the Customer table.

		@return a long integer which represents the SQLCODE returned by UltraLite
	 */
	long OpenCustomerCursor( void );

	/** Retrieve the next row from the Customer table.

		Returns the row after the row which was previously returned by this
		method. If this is the first time this method is called, it will return
		the first row in the Customer table. Ensure that OpenCustomerCursor()
		is called before calling this method for the first time.

		@return a long integer which represents the SQLCODE returned by UltraLite
	 */
	long GetNextCustomerRecord( customer_struct *rec );

	/** Close the cursor which scans through the Customer table.

		This closes the cursor which was previously opened by a call to
		OpenCustomerCursor().

		@return a long integer which represents the SQLCODE returned by UltraLite
	 */
	long CloseCustomerCursor( void );


 private:
	/// The synchronization information which UltraLite uses to connect to Mobilink.
	ul_synch_info _synchInfo;

	/// The username with which to connect to Mobilink.
	TCHAR  _synchUserName[ MAX_SYNCH_USER_NAME_LEN + 1 ];

	/// The version string of the scripts Mobilink should use to synchronize.
	TCHAR  _synchVersion[ MAX_SYNCH_VERSION_LEN + 1 ];

	/// Any connection stream parameters necessary for synchronization.
	TCHAR  _synchStreamParms[ MAX_SYNCH_STREAM_PARMS_LEN + 1 ];
};

#endif
